package com.basebit.api.example;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;

import com.fasterxml.jackson.databind.ObjectMapper;

/**
 * Esta classe serve de exemplo de como acessar a API da basebit usando Java.
 * 
 * Antes de usá-lo, é necessário ter a sua chave e código de acesso à API. Caso ainda
 * não tenha, acesse http://www.basebit.com.br/secure/settings, clique na aba "Trade API" 
 * e pressione o botão "Habilitar". Somente permitimos fazer operação via API se a sua
 * conta estiver configurada com o Google Authenticator.
 * 
 * Após gerar a chave e código de acesso, copie e cole nos campos apropriados baixo.
 * 
 * @author Basebit
 *
 */
public class Example extends Thread {
	
	/**
	 * Substitua abaixo com sua assinatura.
	 */
	private static final String SIGNATURE = "sua assinatura eletrônica";
	
	/**
	 * Substitua abaixo com a chave gerada no site da Basebit.
	 */
	private static final String API_KEY = "sua chave API com 32 caracteres alfanuméricos";
	
	/**
	 * Substitua abaixo com o código gerado no site da Basebit.
	 */
	private static final String API_CODE = "seu código API com 64 caracteres alfanuméricos";
	
	/**
	 * URL base de todas as chamadas que são feitas à API.
	 */
	private static final String BASE_URL = "https://www.basebit.com.br/secure/tapi/";

	/**
	 * Este objeto será usado para desserializar as respostas retornadas em JSON para 
	 * os objetos específicos. 
	 */
	private static final ObjectMapper mapper = new ObjectMapper();
	
	public Example() {
		
		HttpsURLConnection.setDefaultHostnameVerifier(new HostnameVerifier()
        {
            public boolean verify(String hostname, SSLSession session)
            {
                return true;
            }
        });
	}
	
	/**
	 * Calcula o HMAC SHA512 da String "value" usando a chave "key"
	 * @param value A string onde será computado o hmacsha512
	 * @param key Chave para o Sha512
	 * @return O hash calculado (base 16) 
	 * @throws Exception
	 */
	public static String hmacSha512(String value, String key) throws Exception { 
		try { 
			SecretKeySpec keySpec = new SecretKeySpec( key.getBytes("UTF-8"), "HmacSHA512");   
			Mac mac = Mac.getInstance("HmacSHA512"); 
			mac.init(keySpec); 
			byte[] resultBytes = mac.doFinal(value.getBytes("UTF-8"));
			return new BigInteger(1, resultBytes).toString(16);
		} catch (Exception e) { 
			throw e; 
		} 
	}
		
	/**
	 * Executa uma função qualquer da API passando seus parâmetros em um Map<String, Object>.
	 * 
	 * @param function Nome da função a ser executada (ex: listorders).
	 * @param params Map com todos os parâmetros da função.
	 * 
	 * @return resultado da execução em uma String com encoding UTF-8.
	 * 
	 * @throws Exception
	 */
	private final String execute(String function, Map<String, Object> params) throws Exception {
    	
    	String surl =  BASE_URL + function;
		String tohash = surl + SIGNATURE + ((System.currentTimeMillis()/1000));
		String sign = hmacSha512(tohash, API_CODE);
		
		StringBuffer toPost = new StringBuffer("key=" + API_KEY + "&sign=" + sign + "&");
		for (String key : params.keySet()) {
			toPost.append(key).append("=").append(params.get(key)).append("&");
		}
		toPost.deleteCharAt(toPost.length() - 1);
		
		byte[] res = postToURL(surl, toPost.toString());
		return new String(res, "UTF-8");
		
    }
	
	/**
	 * Faz post dos dados já configurados pelo método "execute" acima no endereço da API.
	 * 
	 * @param apiUrl URL da API já contendo a função a ser chamada.
	 * 
	 * @param toPost Dados para serem postados.
	 * 
	 * @return array de bytes com o resultado do post.  
	 * 
	 * @throws Exception
	 */
	private final byte[] postToURL(String apiUrl, String toPost) throws Exception {
		URL url = new URL(apiUrl);
		HttpURLConnection con = (HttpURLConnection) url.openConnection();
		con.setDoInput(true);
		if (toPost != null) {
			con.setDoOutput(true);
			con.getOutputStream().write(toPost.toString().getBytes("UTF-8"));
			con.getOutputStream().close();
		} else {
			con.setDoOutput(false);
		}
		
		int responseCode = con.getResponseCode();
        byte[] result;
        if (responseCode == 200) {
        	result = loadStream(con.getInputStream());
            con.getInputStream().close();
        } else {
        	result = loadStream(con.getErrorStream());
            con.getErrorStream().close();
        }

    	return result;
	}

	/**
	 * Lê dados de um InputStream e retorna um array de bytes com os dados lidos. 
	 * 
	 * @param in O Stream a ser lido.
	 * @return
	 * @throws IOException
	 */
	private final static byte[] loadStream(InputStream in) throws IOException {
        ByteArrayOutputStream o = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        for(;;) {
            int nr = in.read(buffer);

            if (nr == -1)
                break;
            if (nr == 0)
                throw new IOException("Read timed out");

            o.write(buffer, 0, nr);
        }
        return o.toByteArray();
    }
	
	/**
	 * Retorna uma lista com todas as ordens abertas.
	 * 
	 * @return Uma lista com todas as ordens abertas.
	 * @throws Exception
	 */
	protected List<Order> getOpenOrders() throws Exception {
    	Map<String, Object> orderParams = new HashMap<String, Object>();
    	orderParams.put("statusId", 2);
    	String orders = new String(execute("listorders", orderParams));

		GetOrdersResult r = mapper.readValue(orders, GetOrdersResult.class);
		if (r.getErrorMessage() != null) {
			System.out.println("Erro buscando ordens abertas: " + r.getErrorMessage());
			System.exit(-1);
		}
		return r.getOrders();
	}
	
	/**
	 * Rotina principal para testar o programa.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			List<Order> list = new Example().getOpenOrders();
			for (Order order : list) {
				System.out.println("Order ID: " + order.getId());
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	
}
